searchNodes=[{"doc":"A simple extendable DSL evaluator. It's a limited version of Code.eval_string/3 or Code.eval_quoted/3 . So far, the limitations are: No calling module functions; No calling some functions which can cause VM to exit; No sending messages. Here's an example using this module to evaluate a discount number against an order struct: iex&gt; discount_formula = ~s&quot; ...&gt; case order do ...&gt; # old books get a big promotion ...&gt; %{book: %{year: year}} when year &lt; 2000 -&gt; ...&gt; 0.5 ...&gt; ...&gt; %{book: %{tags: tags}} -&gt; ...&gt; # Elixir books! ...&gt; if ~s{elixir} in tags do ...&gt; 0.9 ...&gt; else ...&gt; 1.0 ...&gt; end ...&gt; ...&gt; _ -&gt; ...&gt; 1.0 ...&gt; end ...&gt; &quot; ...&gt; ...&gt; book_order = %{ ...&gt; book : %{ ...&gt; title : &quot;Elixir in Action&quot; , year : 2019 , tags : [ &quot;elixir&quot; ] ...&gt; } ...&gt; } ...&gt; ...&gt; Formular . eval ( discount_formula , [ order : book_order ] ) { :ok , 0.9 } The code being evaluated is just a piece of Elixir code, so it can be expressive when describing business rules. Usage Simple expressions # number iex&gt; Formular . eval ( &quot;1&quot; , [ ] ) { :ok , 1 } # &lt;- note that it&#39;s an integer # plain string iex&gt; Formular . eval ( ~s[&quot;some text&quot;] , [ ] ) { :ok , &quot;some text&quot; } # atom iex&gt; Formular . eval ( &quot;:foo&quot; , [ ] ) { :ok , :foo } # list iex&gt; Formular . eval ( &quot;[:foo, Bar]&quot; , [ ] ) { :ok , [ :foo , Bar ] } Variables Variables can be passed within the binding parameter. # bound value iex&gt; Formular . eval ( &quot;1 + foo&quot; , [ foo : 42 ] ) { :ok , 43 } Functions in the code Kernel functions Kernel functions are limitedly supported. Only a picked list of Kernel functions are supported out of the box so that dangerouse functions such as Kernel.exit/1 will not be invoked. Refer to the code for the whole list. # Kernel function iex&gt; Formular . eval ( &quot;min(5, 100)&quot; , [ ] ) { :ok , 5 } iex&gt; Formular . eval ( &quot;max(5, 100)&quot; , [ ] ) { :ok , 100 } Custom functions Custom functions can be provided in two ways, either in a binding lambda: # bound function iex&gt; Formular . eval ( &quot;1 + add.(-1, 5)&quot; , [ add : &amp; ( &amp;1 + &amp;2 ) ] ) { :ok , 5 } ... or with a context module: iex&gt; defmodule MyContext do ...&gt; def foo ( ) do ...&gt; 42 ...&gt; end ...&gt; end ...&gt; Formular . eval ( &quot;10 + foo&quot; , [ ] , context : MyContext ) { :ok , 52 } Directly calling to module functions in the code are disallowed for security reason. For example: iex&gt; Formular . eval ( &quot;Map.new&quot; , [ ] ) { :error , :no_calling_module_function } iex&gt; Formular . eval ( &quot;min(0, :os.system_time())&quot; , [ ] ) { :error , :no_calling_module_function } Evaluating AST instead of plain string code You may want to use AST instead of string for performance consideration. In this case, an AST can be passed to eval/3 : iex&gt; &quot;a = b = 10; a * b&quot; |&gt; Code . string_to_quoted! ( ) |&gt; Formular . eval ( [ ] ) { :ok , 100 } ...so that you don't have to parse it every time before evaluating it.","ref":"Formular.html","title":"Formular","type":"module"},{"doc":"Evaluate the code with binding context. Parameters code : code to eval. Could be a binary, or parsed AST. binding : the variable binding to support the evaluation options : current these options are supported: context : The module to import before evaluation. Examples iex&gt; Formular . eval ( &quot;1&quot; , [ ] ) { :ok , 1 } iex&gt; Formular . eval ( ~s[&quot;some text&quot;] , [ ] ) { :ok , &quot;some text&quot; } iex&gt; Formular . eval ( &quot;min(5, 100)&quot; , [ ] ) { :ok , 5 } iex&gt; Formular . eval ( &quot;max(5, 100)&quot; , [ ] ) { :ok , 100 } iex&gt; Formular . eval ( &quot;count * 5&quot; , [ count : 6 ] ) { :ok , 30 } iex&gt; Formular . eval ( &quot;add.(1, 2)&quot; , [ add : &amp; ( &amp;1 + &amp;2 ) ] ) { :ok , 3 } iex&gt; Formular . eval ( &quot;Map.new&quot; , [ ] ) { :error , :no_calling_module_function } iex&gt; Formular . eval ( &quot;Enum.count([1])&quot; , [ ] ) { :error , :no_calling_module_function } iex&gt; Formular . eval ( &quot;min(0, :os.system_time())&quot; , [ ] ) { :error , :no_calling_module_function } iex&gt; Formular . eval ( &quot;inspect.(System.A)&quot; , [ inspect : &amp; Kernel . inspect / 1 ] ) { :ok , &quot;System.A&quot; } iex&gt; Formular . eval &quot;f = &amp;IO.inspect/1&quot; , [ ] { :error , :no_calling_module_function } iex&gt; Formular . eval ( &quot;mod = IO; mod.inspect(1)&quot; , [ ] ) { :error , :no_calling_module_function } iex&gt; &quot;a = b = 10; a * b&quot; |&gt; Code . string_to_quoted! ( ) |&gt; Formular . eval ( [ ] ) { :ok , 100 }","ref":"Formular.html#eval/3","title":"Formular.eval/3","type":"function"},{"doc":"","ref":"Formular.html#t:code/0","title":"Formular.code/0","type":"type"},{"doc":"","ref":"Formular.html#t:eval_result/0","title":"Formular.eval_result/0","type":"type"},{"doc":"","ref":"Formular.html#t:option/0","title":"Formular.option/0","type":"type"},{"doc":"","ref":"Formular.html#t:options/0","title":"Formular.options/0","type":"type"}]